import Taro from '@tarojs/taro'
import { getQrCodeParams } from '@utils/qrcode'
import { savePoint, updatePoint } from '@services/common'

function isFunction(fn) {
  return typeof fn === 'function'
}

export function withUtils(opts = {}) {
  let trackEnd;

  return function (OrigComp) {  
    class WrapperComponent extends OrigComp {
      async componentWillMount() {
        if (this.$router.params.scene) {
          this._$_scene = await getQrCodeParams(this.$router.params.scene)
        }
  
        if (super.componentWillMount) {
          super.componentWillMount();
        }
      }
  
      async componentDidShow() {
        // 埋点
        if (isFunction(this._$_track)) {
          const { recordId } = await savePoint(this._$_track())

          trackEnd = () => updatePoint(recordId)
        }
  
        if (super.componentDidShow) {
          super.componentDidShow();
        }
      }
  
      componentDidHide() {
        // 埋点
        if (isFunction(trackEnd)) {
          trackEnd()
        }
  
        if (super.componentDidHide) {
          super.componentDidHide();
        }
      }
  
      onShareAppMessage() {
        const { title, path, image }  = isFunction(this._$shareContent) ? this._$shareContent() : {}

        return {
          title,
          path,
          imageUrl: image,
        }
      }
    }

    return WrapperComponent
  }
}
